from google.colab import _message
import numpy as np

class ex1:
    def __init__(self):
        pass

    def hint(self):
        print("Recuerde que debe ocupar lo mismo que en la parte anterior solo cambiando el argumento")
        print("de solve_ivp para el método de BDF.")

    def check(self):
        pos_ans = [
                    "SOL=SOLVE_IVP(F,(TI,TF),X0,METHOD='BDF',ARGS=ARGS)",
                    'SOL=SOLVE_IVP(F,(TI,TF),X0,METHOD="BDF",ARGS=ARGS)'
        ]
        nbstr = _message.blocking_request('get_ipynb', request='', timeout_sec=10)
        for i in range(len(nbstr['ipynb']["cells"])):
            if nbstr['ipynb']["cells"][i]["source"][0]=="from scipy.integrate import solve_ivp\n":
                ans = nbstr['ipynb']["cells"][i]["source"][-1]
                if ans.replace(" ","").upper() in pos_ans:
                    print("Correcto!")
                else:
                    print("Incorrecto")

    def solution(self):
        print("sol = solve_ivp(F, (ti,tf), X0, method='BDF', args=args)")


import numpy as np

class ex1:
    def __init__(self):
        self.intentos = 0
        pass

    def _RK4(self, f, ti, tf, h, y0, args):
        t = np.arange(ti,tf+h,h)
        N = len(t)
        y_RK4 = np.zeros((N,len(y0)))
        y_RK4[0] = y0
        for i in range(N-1):
            g1 = f(t[i]    , y_RK4[i]       , args)
            g2 = f(t[i]+h/2, y_RK4[i]+h/2*g1, args)
            g3 = f(t[i]+h/2, y_RK4[i]+h/2*g2, args)
            g4 = f(t[i]+h  , y_RK4[i]+h*g3  , args)
            y_RK4[i+1] = y_RK4[i] + h/6*(g1+2*g2+2*g3+g4)
        return (t, y_RK4)

    def hint(self):
        print("Recuerde que para programar RK4 debe calcular primero los g1, g2, g3, g4 y luego")
        print("actualizar la solución. La función de lado derecho recibe los argumentos en el")
        print("siguiente orden: x, C, args.")

    def check(self, y, f, ti, tf, h, y0, args):
        if np.var(y)!=0:
            self.intentos += 1
        correct = True
        t_ref, y_ref = self._RK4(f, ti, tf, h, y0, args)
        if len(y_ref) != len(y):
            print("Incorrecto, el largo de los arreglos no coincide.")
            print()
            correct = False
        else:
            for i in range(len(y_ref)):
                if abs(y[i][0]-y_ref[i][0])>0.01*abs(y_ref[i][0]):
                    print("Incorrecto, los valores no coinciden.")
                    print()
                    correct = False
                    break
                if abs(y[i][1]-y_ref[i][1])>0.01*abs(y_ref[i][1]):
                    print("Incorrecto, los valores no coinciden.")
                    print()
                    correct = False
                    break
                if abs(y[i][2]-y_ref[i][2])>0.01*abs(y_ref[i][2]):
                    print("Incorrecto, los valores no coinciden.")
                    print()
                    correct = False
                    break
                if abs(y[i][3]-y_ref[i][3])>0.01*abs(y_ref[i][3]):
                    print("Incorrecto, los valores no coinciden.")
                    print()
                    correct = False
                    break
        if correct:
            print("Correcto.")
            print()

        return (t_ref, y_ref)

    def solution(self):
        if self.intentos==0:
            print("Debe intentarlo al menos una vez.")
        else:
            print("A continuación, se presenta un código que genera la solución:")
            print()
            print("def RK4(F, xi, xf, h, C0, args):")
            print("    x = np.arange(xi, xf+h, h)")
            print("    N = len(x)")
            print("    C = np.zeros((N,len(C0)))")
            print("    C[0] = C0")
            print("    for i in range(N-1):")
            print("        g1 = F(x[i]    , C[i]       , args)")
            print("        g2 = F(x[i]+h/2, C[i]+h/2*g1, args)")
            print("        g3 = F(x[i]+h/2, C[i]+h/2*g2, args)")
            print("        g4 = F(x[i]+h  , C[i]+h*g3  , args)")
            print("        C[i+1] = C[i] + h/6*(g1+2*g2+2*g3+g4)")
            print("    return (x, C)")


import numpy as np

class ex1:
    def __init__(self):
        self.intentos = 0
        pass

    def _Heun(self,f,zi,zf,h,X0,a,args):
        N = int((zf-zi)/h)
        z = np.linspace(zi, zf, N)
        ndim = len(X0)
        X = np.zeros((N, ndim))
        X[0] = X0
        for i in range(N-1):
            X_tilda = X[i] + h*f(z[i],X[i],args)
            X[i+1] = X[i] + h/2*(f(z[i],X[i],args)+f(z[i+1],X_tilda,args))
            if X[i+1,0] <= a:
                j = i
                break 
        X = X[0:j]
        z = z[0:j]
        return (z,X)

    def hint(self):
        print("Recuerde que para el método de Heun debe calcular primero un paso intermedio, para luego actualizar")
        print("la solución. Además, para este problema debe imponer que si X[:,0] es menor que a,")
        print("entonces el método se detiene y trunque z y X hasta dicha iteración.")
        print("Si obtiene el error: Los largos no coinciden, puede que tenga un problema al truncar las soluciones.")
        print("")

    def check(self, y, f, ti, tf, h, y0, a, args):
        if np.var(y) != 0:
            self.intentos += 1
        correct = True
        t_ref, y_ref = self._Heun(f, ti, tf, h, y0, a, args)
        if abs(len(y_ref)-len(y)) > 1:
            print("Incorrecto, el largo de los arreglos no coincide.")
            print()
            correct = False
        else:
            for i in range(len(y_ref)):
                if abs(y[i][0]-y_ref[i][0])>0.01*abs(y_ref[i][0]) or abs(y[i][1]-y_ref[i][1])>0.01*abs(y_ref[i][1]):
                    print("Incorrecto, los valores no coinciden.")
                    print()
                    correct = False
                    break
        if correct:
            print("Correcto!")
            print()

        return (t_ref, y_ref)

    def solution(self):
        if self.intentos==0:
            print("Debe intentarlo al menos una vez.")
        else:
            print("A continuación, se deja un código que implementa lo pedido.")
            print()
            print("def Heun(f, zi, zf, h, X0, a, args):")
            print("    N = int((zf-zi)/h)")
            print("    z = np.linspace(zi, zf, N)")
            print("    ndim = len(X0)")
            print("    X = np.zeros((N, ndim))")
            print("    X[0] = X0")
            print("    for i in range(N-1):")
            print("        X_tilda = X[i] + h*f(z[i], X[i], args)")
            print("        X[i+1] = X[i] + h/2*(f(z[i], X[i], args)+f(z[i+1], X_tilda, args))")
            print("        if X[i+1,0]<=a:")
            print("            j = i")
            print("            break ")
            print("    X = X[0:j]")
            print("    z = z[0:j]")
            print("    return (z,X)")


import numpy as np

class ex1:
    def __init__(self):
        self.intentos = 0
        pass
    
    def _RK4(self, f, ti, tf, h, y0, args):
        N = int((tf-ti)/h)
        t = np.linspace(ti,tf,N)
        y_RK4 = np.zeros(N)
        y_RK4[0] = y0
        for i in range(N-1):
            g1 = f(t[i]    , y_RK4[i]       , args)
            g2 = f(t[i]+h/2, y_RK4[i]+h/2*g1, args)
            g3 = f(t[i]+h/2, y_RK4[i]+h/2*g2, args)
            g4 = f(t[i]+h  , y_RK4[i]+h*g3  , args)
            y_RK4[i+1] = y_RK4[i] + h/6*(g1+2*g2+2*g3+g4)
        return y_RK4

    def hint(self):
        print("Recuerde que debe imponer la misma condición inicial que las partes anteriores.")
        print("El método de RK4 se basa en calcular g1, g2, g3 y g4, luego calcular el siguiente")
        print("elemento de la solución con la recurrencia conocida.")
        pass 

    def check(self,y,f,ti,tf,h,y0,args):
        if np.var(y)!=0:
            self.intentos += 1
        correct = True
        y_ref = self._RK4(f, ti, tf, h, y0, args)
        if len(y_ref) != len(y):
            print("Incorrecto, el largo de los arreglos no coincide.")
            print()
            correct = False
        else:
            for i in range(len(y_ref)):
                if abs(y[i]-y_ref[i])>0.01*abs(y_ref[i]):
                    print("Incorrecto, los valores no coinciden.")
                    print()
                    correct = False
                    break
        if correct:
            print("Correcto!")
            print()

        return y_ref
        

    def solution(self):
        if self.intentos == 0:
            print("Debe intentarlo al menos una vez.")
        else:
            print("Se plantea el siguiente código como solución:")
            print()
            print("def RK4(f, ti, tf, h, y0, args):")
            print("    N = int((tf-ti)/h)")
            print("    t = np.linspace(ti,tf,N)")
            print("    y_RK4 = np.zeros(N)")
            print("    y_RK4[0] = y0")
            print("    for i in range(N-1):") 
            print("        g1 = f(t[i]    , y_RK4[i]       , args)")  
            print("        g2 = f(t[i]+h/2, y_RK4[i]+h/2*g1, args)")  
            print("        g3 = f(t[i]+h/2, y_RK4[i]+h/2*g2, args)")   
            print("        g4 = f(t[i]+h  , y_RK4[i]+h*g3  , args)")   
            print("        y_RK4[i+1] = y_RK4[i] + h/6*(g1+2*g2+2*g3+g4)")
            print("    return y_RK4")
        pass